/**
  *****************************************************************************************
  * @file src/sys/sysclock.c
  * @author Fedoseev E.A.
  * @date 24.01.17
  * @description          
  * @note  ,       
	*       STM32F4 Reference Manual
  *****************************************************************************************
**/

#include "sysclock.h"                                      //   
#include "sysclock_private.h"                              //     


/*   uint8_t sysclk_config(void)
 *  :         
 *  Note:    .    . 
 */
uint8_t sysclk_config(){
  CLK_CONF CLK_Conf;
	
  CLK_Conf.XTAL      = 8;
  CLK_Conf.CPU_CLOCK = 168;
  CLK_Conf.PLL_M     = CLK_Conf.XTAL;	
  CLK_Conf.PLL_N     = (CLK_Conf.CPU_CLOCK)<<1;
  CLK_Conf.PLL_P     = PLLP_2;
  CLK_Conf.PLL_Q     = 7;
  CLK_Conf.AHB       = AHB_PSC_1;
  CLK_Conf.APB1      = APB_PSC_4;
  CLK_Conf.APB2      = APB_PSC_4;
	
  F4Core_sysclock(&CLK_Conf);
	
  //   8   -1
  RCC->AHB1ENR |= RCC_AHB1ENR_GPIOAEN;
  RCC->CFGR |= RCC_CFGR_MCO1_1;              // HSE selected
	
  GPIOA->MODER |= GPIO_MODER_MODER8_1;       //     
  GPIOA->AFR[1] |= 0<<0;                     // alternate function system
	
  return 0;
}

/*   uint8_t F4Core_sysclock(CLK_CONF* cnf)
 *  : CLK_CONF* cnf -   , 
 *                 
 *  :       cortex-F4
 */
uint8_t F4Core_sysclock(CLK_CONF* cnf){
  F4Core_setFlashLatency(cnf->CPU_CLOCK);
  RCC->APB1ENR |= RCC_APB1ENR_PWREN;                       //   PWR
  RCC->CR |= RCC_CR_HSEON;                                 //    
  while (!(RCC->CR & RCC_CR_HSERDY));                      //   
	
  RCC -> PLLCFGR = 0xF0BC8000;                             //     ,                  	
  RCC -> PLLCFGR |= RCC_PLLCFGR_PLLSRC_HSE;                //       PLL
  F4Core_setPLLM(cnf->PLL_M);                              //   PLL_M
  F4Core_setPLLN(cnf->PLL_N);                              //   PLL_N
  F4Core_setPLLP(cnf->PLL_P);                              //   PLL_P
  F4Core_setPLLQ(cnf->PLL_Q);                              //   PLL_Q
	
  RCC -> CR |= RCC_CR_PLLON;                               //  PLL
  while (!(RCC->CR & RCC_CR_PLLRDY));	                     //   PLL
	
  F4Core_setAHB(cnf->AHB);
  F4Core_setAPB1(cnf->APB1);
  F4Core_setAPB2(cnf->APB2);
	
  RCC -> CFGR |= RCC_CFGR_SW_PLL;                          // c  - PLL
	
  return 0;
}

/*   void F4Core_setPLLM(uint8_t PLL_M)
 *  :    M
 */
void F4Core_setPLLM(uint8_t PLL_M){
  uint8_t M;
  
  if ((PLL_M < 2)|(PLL_M > 63)){                            //   
  	fault_loop();                                           //  ,   
  } else M = PLL_M;
  RCC -> PLLCFGR &= M_CLEAR_MASK;                           //   
  RCC -> PLLCFGR |= M;                                      //   PLLM
  
  return;
}

/*   void F4Core_setPLLN(uint8_t PLL_N)
 *  :    N
 */
void F4Core_setPLLN(uint16_t PLL_N){
  uint16_t N;
	
  if ((PLL_N < 50)|(PLL_N > 432)){                          //    
    fault_loop();                                           //  ,   
  } else N = PLL_N;
  RCC -> PLLCFGR &= N_CLEAR_MASK;                           //   	
  RCC -> PLLCFGR |= N<<6;                                   //   PLLN
	
  return;
}

/*   void F4Core_setPLLP(uint8_t PLL_P)
 *  :    P
 */
void F4Core_setPLLP(uint8_t PLL_P){
  uint8_t P;
  
  if ((PLL_P != PLLP_2)&(PLL_P != PLLP_4)
  	 &(PLL_P != PLLP_6)&(PLL_P != PLLP_8)){                 //    
  	fault_loop();                                           //  ,   
  } else P = PLL_P;
  RCC -> PLLCFGR &= P_CLEAR_MASK;                           //   
  RCC -> PLLCFGR |= P<<16;                                  //   PLLP
  
  return;
}

/*   void F4Core_setPLLQ(uint8_t PLL_Q)
 *  :    Q
 */
void F4Core_setPLLQ(uint8_t PLL_Q){
  uint8_t Q;
  
  if ((PLL_Q < 2)|(PLL_Q > 15)){                            //   
  	fault_loop();                                           //  ,   
  } else Q = PLL_Q;
  RCC -> PLLCFGR &= Q_CLEAR_MASK;                           //   
  RCC -> PLLCFGR |= Q<<24;                                  //   PLLQ
  
  return;
}

/*   void F4Core_setAPB1(uint8_t APB1)
 *  :     APB1
 */
void F4Core_setAPB1(uint8_t APB1){
  RCC -> CFGR &= APB1_CLEAR_MASK;                           //   
  RCC -> CFGR |= (APB1&0x7)<<10;                            //    
	
  return;
}
	
/*   void F4Core_setAPB2(uint8_t APB2)
 *  :     APB2
 */
void F4Core_setAPB2(uint8_t APB2){
  RCC -> CFGR &= APB2_CLEAR_MASK;                           //   
  RCC -> CFGR |= (APB2&0x7)<<13;                            //    
	
  return;
}

/*   void F4Core_setAHB(uint8_t AHB)
 *  :     AHB
 */
void F4Core_setAHB(uint8_t AHB){
  RCC -> CFGR &= AHB_CLEAR_MASK;                            //   
  RCC -> CFGR |= (AHB&0xF)<<4;                              //    
	
  return;
}

/*   void F4Core_setFlashLatency(uint8_t CPU_CLOCK)
 *  :   flash'a       
 */
void F4Core_setFlashLatency(uint8_t CPU_CLOCK){
  uint8_t buf = 0;
  uint8_t WS = 0;                                           // Wait State -   flash'a 
  
  while(buf<CPU_CLOCK){
    buf += 30;
    if (buf < CPU_CLOCK) WS++;
  }
  
  FLASH -> ACR &= ACR_CLEAR_MASK;                           //   
  FLASH -> ACR |= WS;                                       //   
  
  return;
}

/*   void fault_loop()
 *  :  .      
 */
void fault_loop(void){
	while(1);
}


/*   void RESET(void)
 *  :    
 */
void PRG_RESET(void)
{
  SCB->AIRCR = 0x05FA0004;
  
  return;
}

