#ifndef _SYS_CLOCK_PRIVATE_H
#define _SYS_CLOCK_PRIVATE_H

//____________________INCLUDE_____________________//
  #include "stm32f407xx.h"

//________________TYPES DEFINITION________________//
  typedef struct{
		uint8_t XTAL;                 //   (4-26 )
		uint8_t CPU_CLOCK;            //     (168  )
		uint8_t PLL_M;                //   (2-63)
		uint16_t PLL_N;               //   (50-432)
		uint8_t PLL_P;                //   (2, 4, 6, 8)
		uint8_t PLL_Q;                //   (2-15)
		uint8_t AHB;                  //     (1, 2, 4, 8, 16, 64, 128, 256, 512)
		uint8_t APB1;                 //     (1, 2, 4, 8, 16)
		uint8_t APB2;                 //     (1, 2, 4, 8, 16)
		
	} CLK_CONF;
		
//_______________FUNCTION PROTOTYPE_______________//
  uint8_t F4Core_sysclock(CLK_CONF* cnf);
  void F4Core_setPLLM(uint8_t PLL_M);
  void F4Core_setPLLN(uint16_t PLL_N);
  void F4Core_setPLLP(uint8_t PLL_P);
  void F4Core_setPLLQ(uint8_t PLL_Q);
  void F4Core_setAPB1(uint8_t APB1);
  void F4Core_setAPB2(uint8_t APB2);
  void F4Core_setAHB(uint8_t AHB);
  void F4Core_setFlashLatency(uint8_t CPU_CLOCK);
  void fault_loop(void);
	
//______________________ENUM______________________//
	enum{
		PLLP_2 = 0,
		PLLP_4 = 1,
		PLLP_6 = 2,
		PLLP_8 = 3,
	} PLLP_DIP;
	enum{
		APB_PSC_1  = 0,
		APB_PSC_2  = 0x04,
		APB_PSC_4  = 0x05,
		APB_PSC_8  = 0x06,
		APB_PSC_16 = 0x07,
		
	} APB_DIV;
  
	enum{
		AHB_PSC_1   = 0,
		AHB_PSC_2   = 0x08,
		AHB_PSC_4   = 0x09,
		AHB_PSC_8   = 0x0A,
		AHB_PSC_16  = 0x0B,
		AHB_PSC_64  = 0x0C,
		AHB_PSC_128 = 0x0D,
		AHB_PSC_256 = 0x0E,
		AHB_PSC_512 = 0x0F,
		
	} AHB_DIV;
	
//_________________CLEAR MASK___________________//
	#define AHB_CLEAR_MASK  ((uint32_t)0xFFFFFF0F)
	#define APB1_CLEAR_MASK ((uint32_t)0xFFFFE3FF)
	#define APB2_CLEAR_MASK ((uint32_t)0xFFFF1FFF)
	#define M_CLEAR_MASK    ((uint32_t)0xFFFFFFC0)
	#define N_CLEAR_MASK    ((uint32_t)0xFFFF803F)
	#define P_CLEAR_MASK    ((uint32_t)0xFFFCFFFF)
        #define Q_CLEAR_MASK    ((uint32_t)0xF0FFFFFF)
	#define ACR_CLEAR_MASK  ((uint32_t)0xFFFFFFF8)
	
#endif //_SYS_CLOCK_PRIVATE_H
